import { getEnv, ENV_TYPE } from './util'
import { navigateBack, navigateTo, redirectTo } from './router'
import { createSelectorQuery } from './api/createSelectorQuery'

const env = getEnv()

const RequestQueue = {
  MAX_REQUEST: 5,
  queue: [],
  request (options) {
    this.push(options)
    this.run()
  },

  push (options) {
    this.queue.push(options)
  },

  run () {
    if (!this.queue.length) {
      return
    }
    if (this.queue.length <= this.MAX_REQUEST) {
      let options = this.queue.shift()
      let completeFn = options.complete
      options.complete = () => {
        completeFn && completeFn.apply(options, [...arguments])
        this.run()
      }
      wx.request(options)
    }
  }
}

function request (options) {
  options = options || {}
  if (typeof options === 'string') {
    options = {
      url: options
    }
  }
  if (env === ENV_TYPE.WEAPP) {
    const p = new Promise((resolve, reject) => {
      options['success'] = res => {
        resolve(res.data)
      }
      options['fail'] = res => {
        reject(res)
      }
      RequestQueue.request(options)
    })
    return p
  } else {
    const url = options.url
    const params = {}
    params.body = options.data
    params.headers = options.header
    params.method = options.method
    params.mode = options.mode
    params.credentials = options.credentials
    params.cache = options.cache
    return fetch(url, params)
      .then(response => {
        if (options.responseType === 'arraybuffer') {
          return response.arrayBuffer()
        }
        if (options.dataType === 'json' || typeof options.dataType === 'undefined') {
          return response.json()
        }
        if (options.responseType === 'text') {
          return response.text()
        }
      })
  }
}

function processApis (taro) {
  const onAndSyncApis = {
    onSocketOpen: true,
    onSocketError: true,
    onSocketMessage: true,
    onSocketClose: true,
    onBackgroundAudioPlay: true,
    onBackgroundAudioPause: true,
    onBackgroundAudioStop: true,
    onNetworkStatusChange: true,
    onAccelerometerChange: true,
    onCompassChange: true,
    onBluetoothAdapterStateChange: true,
    onBluetoothDeviceFound: true,
    onBLEConnectionStateChange: true,
    onBLECharacteristicValueChange: true,
    onBeaconUpdate: true,
    onBeaconServiceChange: true,
    onUserCaptureScreen: true,
    onHCEMessage: true,
    onGetWifiList: true,
    onWifiConnected: true,
    setStorageSync: true,
    getStorageSync: true,
    getStorageInfoSync: true,
    removeStorageSync: true,
    clearStorageSync: true,
    getSystemInfoSync: true,
    getExtConfigSync: true
  }
  const noPromiseApis = {
    // 媒体
    stopRecord: true,
    getRecorderManager: true,
    pauseVoice: true,
    stopVoice: true,
    pauseBackgroundAudio: true,
    stopBackgroundAudio: true,
    getBackgroundAudioManager: true,
    createAudioContext: true,
    createInnerAudioContext: true,
    createVideoContext: true,
    createCameraContext: true,

    // 位置
    createMapContext: true,

    // 设备
    canIUse: true,
    startAccelerometer: true,
    stopAccelerometer: true,
    startCompass: true,
    stopCompass: true,

    // 界面
    hideToast: true,
    hideLoading: true,
    showNavigationBarLoading: true,
    hideNavigationBarLoading: true,
    createAnimation: true,
    pageScrollTo: true,
    createSelectorQuery: true,
    createCanvasContext: true,
    createContext: true,
    drawCanvas: true,
    hideKeyboard: true,
    stopPullDownRefresh: true,

    // 拓展接口
    arrayBufferToBase64: true,
    base64ToArrayBuffer: true,

    getUpdateManager: true,
    createWorker: true
  }
  const otherApis = {
    // 网络
    uploadFile: true,
    downloadFile: true,
    connectSocket: true,
    sendSocketMessage: true,
    closeSocket: true,

    // 媒体
    chooseImage: true,
    previewImage: true,
    getImageInfo: true,
    saveImageToPhotosAlbum: true,
    startRecord: true,
    playVoice: true,
    getBackgroundAudioPlayerState: true,
    playBackgroundAudio: true,
    seekBackgroundAudio: true,
    chooseVideo: true,
    saveVideoToPhotosAlbum: true,

    // 文件
    saveFile: true,
    getFileInfo: true,
    getSavedFileList: true,
    getSavedFileInfo: true,
    removeSavedFile: true,
    openDocument: true,

    // 数据缓存
    setStorage: true,
    getStorage: true,
    getStorageInfo: true,
    removeStorage: true,
    clearStorage: true,

    // 位置
    getLocation: true,
    chooseLocation: true,
    openLocation: true,

    // 设备
    getSystemInfo: true,
    getNetworkType: true,
    makePhoneCall: true,
    scanCode: true,
    setClipboardData: true,
    getClipboardData: true,
    openBluetoothAdapter: true,
    closeBluetoothAdapter: true,
    getBluetoothAdapterState: true,
    startBluetoothDevicesDiscovery: true,
    stopBluetoothDevicesDiscovery: true,
    getBluetoothDevices: true,
    getConnectedBluetoothDevices: true,
    createBLEConnection: true,
    closeBLEConnection: true,
    getBLEDeviceServices: true,
    getBLEDeviceCharacteristics: true,
    readBLECharacteristicValue: true,
    writeBLECharacteristicValue: true,
    notifyBLECharacteristicValueChange: true,
    startBeaconDiscovery: true,
    stopBeaconDiscovery: true,
    getBeacons: true,
    setScreenBrightness: true,
    getScreenBrightness: true,
    setKeepScreenOn: true,
    vibrateLong: true,
    vibrateShort: true,
    addPhoneContact: true,
    getHCEState: true,
    startHCE: true,
    stopHCE: true,
    sendHCEMessage: true,
    startWifi: true,
    stopWifi: true,
    connectWifi: true,
    getWifiList: true,
    setWifiList: true,
    getConnectedWifi: true,

    // 界面
    showToast: true,
    showLoading: true,
    showModal: true,
    showActionSheet: true,
    setNavigationBarTitle: true,
    setNavigationBarColor: true,
    setTabBarBadge: true,
    removeTabBarBadge: true,
    showTabBarRedDot: true,
    hideTabBarRedDot: true,
    setTabBarStyle: true,
    setTabBarItem: true,
    showTabBar: true,
    hideTabBar: true,
    setTopBarText: true,
    switchTab: true,
    reLaunch: true,
    startPullDownRefresh: true,

    // 第三方平台
    getExtConfig: true,

    // 开放接口
    login: true,
    authorize: true,
    getUserInfo: true,
    requestPayment: true,
    showShareMenu: true,
    hideShareMenu: true,
    updateShareMenu: true,
    getShareInfo: true,
    chooseAddress: true,
    addCard: true,
    openCard: true,
    openSetting: true,
    getSetting: true,
    getWeRunData: true,
    navigateToMiniProgram: true,
    navigateBackMiniProgram: true,
    chooseInvoiceTitle: true,
    checkIsSupportSoterAuthentication: true,
    startSoterAuthentication: true,
    checkIsSoterEnrolledInDevice: true
    //
  }
  const weApis = Object.assign(onAndSyncApis, noPromiseApis, otherApis)
  Object.keys(weApis).forEach(key => {
    if (env === ENV_TYPE.WEAPP) {
      if (!onAndSyncApis[key] && !noPromiseApis[key]) {
        taro[key] = options => {
          let task = null
          let obj = {}
          options = options || {}
          if (typeof options === 'string') {
            return wx[key](options)
          }
          const p = new Promise((resolve, reject) => {
            ['fail', 'success', 'complete'].forEach((k) => {
              obj[k] = (res) => {
                if (k === 'success') {
                  if (key === 'connectSocket') {
                    resolve(task)
                  } else {
                    resolve(res)
                  }
                } else if (k === 'fail') {
                  reject(res)
                }
              }
            })
            task = wx[key](obj)
          })
          if (key === 'uploadFile' || key === 'downloadFile') {
            p.progress = cb => {
              task.onProgressUpdate(cb)
              return p
            }
            p.abort = cb => {
              cb && cb()
              task.abort()
              return p
            }
          }
          return p
        }
      } else {
        taro[key] = (...args) => {
          return wx[key].apply(wx, args)
        }
      }
    } else {
      taro[key] = () => {
        console.log(`暂时不支持 ${key}`)
      }
    }
  })
}

export default function initNativeApi (taro) {
  processApis(taro)
  taro.request = request
  taro.navigateTo = navigateTo
  taro.navigateBack = navigateBack
  taro.redirectTo = redirectTo
  taro.createSelectorQuery = createSelectorQuery
}
